require 'uu/os/cmd/command_client'
require 'uu/os/security/session'

module UU
  module C3

    # Client of service providing operations for self-service deployment of uuApps.
    class ResourceGroup

      # Remote service path.
      # @private
      PATH = 'uu-c3/ResourceGroup'

      # Default URLs where common commands are running.
      # @private
      DEFAULT_COMMON_BASE_URL = /https:\/\/(api|cmd)\.plus4u\.net/

      # Default URL for C3 (and other syscalls).
      # @private
      DEFAULT_C3_BASE_URL = 'https://sys.plus4u.net'

      # Try to auto-configure client base URL.
      ext_config = UU::OS::CMD::CommandClient.const_get(:EXTERNAL_CONFIGURATION)
      base_url = ext_config.get(:base_url, PATH)
      if base_url.nil? || base_url =~ DEFAULT_COMMON_BASE_URL
        UU::OS::CMD::CommandClient.load_configuration({'uu.c3.base_url' => DEFAULT_C3_BASE_URL})
      end
      # Command is not yet invoked through uuGateway, therefore do not append routing data.
      UU::OS::CMD::CommandClient.load_configuration({'uu.c3.append_routing_data' => 'false'})

      # Creates a new instance of +ResourceGroup+.
      #
      # @param session [UU::OS::Security::Session] Instance of session representing authenticated user
      #   (can be obtained with {UU::OS::Swecurity::Session#current_session} method).
      def initialize(session = nil)
        @client = UU::OS::CMD::CommandClient.new(PATH, session)
      end

      # @note This command expects that all applications in all ResourcePools of this ResourceGroup
      #   are deployed in both slots and are prepared for {AppDeployment#swap}. I.e. they are properly
      #   scaled.
      # @note Ignores applications in +UNDEPLOYED+ state.
      # @note This command cannot be called unless there is a +slotMapping+ attribute set on ResourceGroup.
      #   The value is irrelevant as long as it is not +nil+.
      # @note Don't call any other commands which causes updates (i.e. deploy, undeploy, redeploy, share, unshare, scale)
      #   until asynchronous part of this command finishes.
      # @note The applications are first swapped in uuCloudg01OperationRegistry and all the gateways are updated at the
      #   end at once. So until the command finishes the applications might not be available.
      # Swaps all applications in all ResourcePools of this ResourceGroup between +PRODUCTION+ and +BETA+ slots.
      # @param resource_group_uri [UU::OS::UESURI, String] URI of ResourceGroup.
      # @return [UU::OS::Lang::Future] Future instance. Make sure than asynchronous part ends successfully.
      def swap(resource_group_uri)
        @client.invoke(:swap, resource_group_uri, parameters: {})
      end

      # @note This command expects that all applications in all ResourcePools of this ResourceGroup
      #   are deployed in both slots and are prepared for {AppDeployment#swap}. I.e. they are properly
      #   scaled.
      # @note Ignores applications in +UNDEPLOYED+ state.
      # @note This command cannot be called unless there is a +slotMapping+ attribute set on ResourceGroup.
      #   The value is irrelevant as long as it is not +nil+.
      # @note Don't call any other commands which causes updates (i.e. deploy, undeploy, redeploy, share, unshare, scale)
      #   until asynchronous part of this command finishes.
      # @note The applications are first swapped in uuCloudg01OperationRegistry and all the gateways are updated at the
      #   end at once. So until the command finishes the applications might not be available.
      # Swaps all applications in all ResourcePools of this ResourceGroup between +PRODUCTION+ and +BETA+ slots.
      # @param resource_group_uri [UU::OS::UESURI, String] URI of ResourceGroup.
      def self.swap(resource_group_uri)
        self.new(UU::OS::Security::Session.current_session).swap(resource_group_uri)
      end

    end

  end
end

